@@ -7,14 +7,14 @@ FuzzPy: Fuzzy sets for Python
 @license: LGPL-3
 """
 
-__version__ = (0, 4, 0)
+__version__ = (0, 4, 1)
 
 __all__ = ['iset', 'fset', 'fnumber', 'graph', 'fgraph', 'visualization']
 __name__ = 'fuzz'
 
-from iset import *
-from fset import *
-from fnumber import *
-from graph import *
-from fgraph import *
-from visualization import *
+from .iset import *
+from .fset import *
+from .fnumber import *
+from .graph import *
+from .fgraph import *
+from .visualization import *
