@@ -10,7 +10,7 @@ Fuzzy number module. Contains basic fuzzy number class definitions.
 from math import e, sqrt, log
 from numbers import Number
 
-from fset import FuzzySet
+from .fset import FuzzySet
 
 
 class RealRange(tuple):
@@ -23,12 +23,12 @@ class RealRange(tuple):
         before returning the range object.
         """
         if not len(arg) == 2:
-            raise ValueError("range must consist of two values")
+            raise ValueError('range must consist of two values')
         if not isinstance(arg[0], Number) \
         or not isinstance(arg[1], Number):
-            raise TypeError("range values must be numeric")
+            raise TypeError('range values must be numeric')
         if arg[0] > arg[1]:
-            raise ValueError("range may not have negative size")
+            raise ValueError('range may not have negative size')
         return tuple.__new__(cls, arg)
 
     @property
@@ -83,7 +83,7 @@ class RealRange(tuple):
         @rtype: C{bool}
         """
         if not isinstance(other, RealRange):
-            raise TypeError("argument must be a RealRange")
+            raise TypeError('argument must be a RealRange')
         if other[0] <= self[0] and other[1] >= self[1]:
             return True
         return False
@@ -98,7 +98,7 @@ class RealRange(tuple):
         @rtype: C{bool}
         """
         if not isinstance(other, RealRange):
-            raise TypeError("argument must be a RealRange")
+            raise TypeError('argument must be a RealRange')
         if self[0] <= other[0] and self[1] >= other[1]:
             return True
         return False
@@ -138,7 +138,7 @@ class FuzzyNumber(object):
         Constructor. Not to be instantiated directly.
         """
         if self.__class__ is FuzzyNumber:
-            raise NotImplementedError("please use one of the subclasses")
+            raise NotImplementedError('please use one of the subclasses')
 
     def __repr__(self):
         """\
@@ -167,14 +167,14 @@ class FuzzyNumber(object):
         @param value: A value in the universal set.
         @type value: C{float}
         """
-        raise NotImplementedError("mu method must be overridden")
+        raise NotImplementedError('mu method must be overridden')
 
     def normalize(self):
         """\
         Normalize this fuzzy number, so that its height is equal to 1.0.
         """
         if not self.height == 1.0:
-            raise NotImplementedError("normalize method must be overridden")
+            raise NotImplementedError('normalize method must be overridden')
 
     kernel = None
     support = None
@@ -193,12 +193,12 @@ class PolygonalFuzzyNumber(FuzzyNumber):
         @type points: C{list} of C{tuple}
         """
         if not points[0][1] == 0.0 or not points[-1][1] == 0.0:
-            raise ValueError("points must start and end with mu = 0")
+            raise ValueError('points must start and end with mu = 0')
         for i in range(1, len(points)):
             if not points[i][0] >= points[i - 1][0]:
-                raise ValueError("points must be in increasing order")
+                raise ValueError('points must be in increasing order')
         self.points = points
-        FuzzyNumber.__init__(self)
+        super(PolygonalFuzzyNumber, self).__init__()
 
     @staticmethod
     def _binary_sanity_check(other):
@@ -210,8 +210,8 @@ class PolygonalFuzzyNumber(FuzzyNumber):
         @type other: L{PolygonalFuzzyNumber}
         """
         if not isinstance(other, PolygonalFuzzyNumber):
-            raise TypeError("binary operation only permitted between \
-                               polygonal fuzzy numbers")
+            raise TypeError('operation only permitted between polygonal '
+                            'fuzzy numbers')
 
     def mu(self, value):
         """\
@@ -329,10 +329,10 @@ class PolygonalFuzzyNumber(FuzzyNumber):
         points = []
         for i in range(len(self.points)):
             if self.points[i][1] >= other.mu(self.points[i][0]):
-                points.append((self.points[i], self, i))
+                points.append((self.points[i], i, self))
         for i in range(len(other.points)):
             if other.points[i][1] >= self.mu(other.points[i][0]):
-                points.append((other.points[i], other, i))
+                points.append((other.points[i], i, other))
         points.sort()
         i = 0
         while(True):
@@ -340,10 +340,10 @@ class PolygonalFuzzyNumber(FuzzyNumber):
                 if points[i][0][0] == points[i + 1][0][0]:
                     del points[i]
                     continue
-                if points[i][1] is not points[i + 1][1]:
+                if points[i][2] is not points[i + 1][2]:
                     points.insert(i + 1, (self._line_intersection(points[i][0],
-                        points[i][1].points[points[i][2] + 1], points[i + 1][0],
-                        points[i + 1][1].points[points[i + 1][2] - 1]),
+                        points[i][2].points[points[i][1] + 1], points[i + 1][0],
+                        points[i + 1][2].points[points[i + 1][1] - 1]),
                         None, None))
                     if points[i + 1][0][0] == points[i + 2][0][0]:
                         del points[i + 1]
@@ -392,10 +392,10 @@ class PolygonalFuzzyNumber(FuzzyNumber):
         points = []
         for i in range(len(self.points)):
             if self.points[i][1] <= other.mu(self.points[i][0]):
-                points.append((self.points[i], self, i))
+                points.append((self.points[i], i, self))
         for i in range(len(other.points)):
             if other.points[i][1] <= self.mu(other.points[i][0]):
-                points.append((other.points[i], other, i))
+                points.append((other.points[i], i, other))
         points.sort()
         i = 0
         while(True):
@@ -404,10 +404,10 @@ class PolygonalFuzzyNumber(FuzzyNumber):
                 or points[i][0][1] == 0.0 and points[i + 1][0][1] == 0.0:
                     del points[i]
                     continue
-                if points[i][1] is not points[i + 1][1]:
+                if points[i][2] is not points[i + 1][2]:
                     points.insert(i + 1, (self._line_intersection(points[i][0],
-                        points[i][1].points[points[i][2] + 1], points[i + 1][0],
-                        points[i + 1][1].points[points[i + 1][2] - 1]),
+                        points[i][2].points[points[i][1] + 1], points[i + 1][0],
+                        points[i + 1][2].points[points[i + 1][1] - 1]),
                         None, None))
                     i += 1
                 i += 1
@@ -456,13 +456,13 @@ class TrapezoidalFuzzyNumber(FuzzyNumber):
         """
         if not (isinstance(kernel, tuple) and len(kernel) == 2) \
         or not (isinstance(support, tuple) and len(support) == 2):
-            raise TypeError("kernel and support must be 2-tuples")
+            raise TypeError('kernel and support must be 2-tuples')
         self.kernel = RealRange(kernel)
         self.support = RealRange(support)
         if not self.kernel <= self.support:
-            raise ValueError("kernel range must be within support range")
+            raise ValueError('kernel range must be within support range')
         self.height = 1.0
-        FuzzyNumber.__init__(self)
+        super(TrapezoidalFuzzyNumber, self).__init__()
 
     @property
     def triangular(self):
@@ -483,8 +483,8 @@ class TrapezoidalFuzzyNumber(FuzzyNumber):
         @type other: L{TrapezoidalFuzzyNumber}
         """
         if not isinstance(other, TrapezoidalFuzzyNumber):
-            raise TypeError("binary operation only permitted between \
-                             trapezoidal fuzzy numbers")
+            raise TypeError('operation only permitted between trapezoidal '
+                            'fuzzy numbers')
 
     def __add__(self, other):
         """\
@@ -572,7 +572,8 @@ class TriangularFuzzyNumber(TrapezoidalFuzzyNumber):
         @param support: The support of the fuzzy number.
         @type support: C{tuple}
         """
-        TrapezoidalFuzzyNumber.__init__((kernel, kernel), support)
+        super(TriangularFuzzyNumber, self).__init__(kernel=(kernel, kernel),
+            support=support)
 
 
 class GaussianFuzzyNumber(FuzzyNumber):
@@ -591,8 +592,47 @@ class GaussianFuzzyNumber(FuzzyNumber):
         self.mean = mean
         self.stddev = stddev
         self.height = 1.0
-        FuzzyNumber.__init__(self)
+        super(GaussianFuzzyNumber, self).__init__()
 
+    @staticmethod
+    def _binary_sanity_check(other):
+        """\
+        Check that the other argument to a binary operation is also a
+        gaussian fuzzy number, raising a TypeError otherwise.
+
+        @param other: The other argument.
+        @type other: L{GaussianFuzzyNumber}
+        """
+        if not isinstance(other, GaussianFuzzyNumber):
+            raise TypeError('operation only permitted between Gaussian '
+                            'fuzzy numbers')
+
+    def __add__(self, other):
+        """\
+        Addition operation.
+
+        @param other: The other gaussian fuzzy number.
+        @type other: L{GaussianFuzzyNumber}
+        @return: Sum of the gaussian fuzzy numbers.
+        @rtype: L{GaussianFuzzyNumber}
+        """
+        self._binary_sanity_check(other)
+        return self.__class__(self.mean + other.mean,
+                              self.stddev + other.stddev)
+
+    def __sub__(self, other):
+        """\
+        Subtraction operation.
+        
+        @param other: The other gaussian fuzzy number.
+        @type other: L{GaussianFuzzyNumber}
+        @return: Difference of the gaussian fuzzy numbers.
+        @rtype: L{GaussianFuzzyNumber}
+        """
+        self._binary_sanity_check(other)
+        return self.__class__(self.mean - other.mean,
+                              self.stddev + other.stddev)
+    
     def mu(self, value):
         """\
         Return the membership level of a value in the universal set domain of
@@ -601,9 +641,8 @@ class GaussianFuzzyNumber(FuzzyNumber):
         @param value: A value in the universal set.
         @type value: C{float}
         """
-        if value not in self.support:
-            return 0.0
-        return e ** -((value - self.mean) ** 2 / (2.0 * self.stddev ** 2))
+        return e ** -((value - self.mean) ** 2 / (2.0 * self.stddev ** 2)) \
+            if value in self.support else 0.0
 
     @property
     def kernel(self):
@@ -651,7 +690,7 @@ class GaussianFuzzyNumber(FuzzyNumber):
         @rtype: L{PolygonalFuzzyNumber}
         """
         if np < 0:
-            raise ValueError("number of points must be positive")
+            raise ValueError('number of points must be positive')
         points = []
         start, end = self.support
         increment = (self.mean - start) / float(np + 1)
@@ -664,5 +703,4 @@ class GaussianFuzzyNumber(FuzzyNumber):
             value = self.mean + i * increment
             points.append((value, self.mu(value)))
         points.append((end, 0.0))
-        print points
         return PolygonalFuzzyNumber(points)
